#include <mciReader_hsd_root.h>

ClassImp(mciReader_hsd_root);

mciReader_hsd_root::mciReader_hsd_root(/* args */) : is_init(false), fCurrentEvent(-1), fEvent(nullptr), fParticle(nullptr)
{
}

mciReader_hsd_root::~mciReader_hsd_root()
{
}

Bool_t mciReader_hsd_root::ChainCheck()
{
  if (!is_init)
  {
    return false;
  }

  if (fCurrentEvent == -1)
  {
    return false;
  }

  if (!fChainPHSD->GetEntry(fCurrentEvent))
  {
    return false;
  }
  if (!fChainFrag->GetEntry(fCurrentEvent))
  {
    return false;
  }

  return true;
}

void mciReader_hsd_root::SetChain(const TString &inputFileName)
{
  fChainPHSD = (TChain *)mciUtility::GetInstance()->initChain(inputFileName, fChainPHSDName);

  fChainPHSD->SetBranchAddress("event", &fEvent);

  fChainFrag = (TChain *)mciUtility::GetInstance()->initChain(inputFileName, fChainFragName);

  fChainFrag->SetBranchAddress("eventFrag", &fEventFrag);

  is_init = kTRUE;
}

mciEvent *mciReader_hsd_root::ReadEvent(Long64_t iev)
{
  mciEvent *event = new mciEvent();

  fCurrentEvent = iev;

  if (!ChainCheck())
  {
    return event;
  }

  event->SetB(fEvent->GetB());
  event->SetPhiRP(0.); // in PHQMD axis orientation is rotated by 180 deg. In HSD/PHSD it's ok.
  //event->SetPhiRP(TMath::Pi()); // in PHQMD axis orientation is rotated by 180 deg. In HSD/PHSD it's ok.
  event->SetNparticles(fEvent->GetNpart() + fEventFrag->GetNfragMST()); // number of particles from PHSD and number of MST fragments

  return event;
}

mciParticle *mciReader_hsd_root::ReadParticle(Int_t ipart)
{
  mciParticle *particle = new mciParticle();

  if (!ChainCheck())
  {
    return particle;
  }

  if (ipart >= fEvent->GetNpart() + fEventFrag->GetNfragMST())
  {
    return particle;
  }

  if (ipart < 0)
  {
    return particle;
  }

  bool is_particle = false;
  bool is_fragment = false;

  if (ipart < fEvent->GetNpart())
  {
    is_particle = true;
  }
  if (ipart >= fEvent->GetNpart())
  {
    is_fragment = true;
  }

  if (!is_particle && !is_fragment)
  {
    return particle;
  }

  if (is_particle && is_fragment)
  {
    return particle;
  }

  if (is_particle)
  {
    fParticle = fEvent->GetParticle(ipart);
    if (fParticle->IsInMST()) // Check whether the particle from PHSD was used to create MST fragment
    {
      return particle;
    }
    particle->SetEnergy(fParticle->E());
    particle->SetPdg(fParticle->GetPdg());
    particle->SetPxPyPz(fParticle->Px(), fParticle->Py(), fParticle->Pz());
    particle->SetTime(0.);
    particle->SetXYZ(0., 0., 0.);
    particle->SetCharge(fParticle->GetCharge());
  }
  if (is_fragment)
  {
    fFragmentMST = fEventFrag->GetFragmentMST(ipart - fEvent->GetNpart());
    particle->SetEnergy(fFragmentMST->E());
    particle->SetPdg(fFragmentMST->GetPdg());
    particle->SetPxPyPz(fFragmentMST->Px(), fFragmentMST->Py(), fFragmentMST->Pz());
    particle->SetTime(0.);
    particle->SetXYZ(0., 0., 0.);
    particle->SetCharge(fFragmentMST->GetZ());
  }

  return particle;
}
