#include <mciReader_mcpico.h>

ClassImp(mciReader_mcpico);

mciReader_mcpico::mciReader_mcpico(/* args */) : is_init(false), fCurrentEvent(-1)
{
}

mciReader_mcpico::~mciReader_mcpico()
{
}

Bool_t mciReader_mcpico::ChainCheck()
{
  if (!is_init)
  {
    return false;
  }

  if (fCurrentEvent == -1)
  {
    return false;
  }

  if (!fChain->GetEntry(fCurrentEvent))
  {
    return false;
  }

  return true;
}

void mciReader_mcpico::SetChain(const TString &inputFileName)
{
  fChain = (TChain *)mciUtility::GetInstance()->initChain(inputFileName, fChainName);

  fChain->SetBranchAddress("bimp", &bimp);
  fChain->SetBranchAddress("phi2", &phi2);
  fChain->SetBranchAddress("phi3", &phi3);
  fChain->SetBranchAddress("ecc2", &ecc2);
  fChain->SetBranchAddress("ecc3", &ecc3);
  fChain->SetBranchAddress("npart", &npart);
  fChain->SetBranchAddress("nh", &nh);
  fChain->SetBranchAddress("momx", momx);
  fChain->SetBranchAddress("momy", momy);
  fChain->SetBranchAddress("momz", momz);
  fChain->SetBranchAddress("ene", ene);
  fChain->SetBranchAddress("hid", hid);
  fChain->SetBranchAddress("pdg", pdg);
  fChain->SetBranchAddress("charge", charge);

  is_init = kTRUE;
}

mciEvent *mciReader_mcpico::ReadEvent(Long64_t iev)
{
  mciEvent *event = new mciEvent();
  fCurrentEvent = iev;

  if (!ChainCheck())
  {
    return event;
  }

  event->SetB(bimp);
  event->SetPhiRP(phi2);
  event->SetNparticles(nh);
  event->SetNparticipants(npart);

  return event;
}

mciParticle *mciReader_mcpico::ReadParticle(Int_t ipart)
{
  mciParticle *particle = new mciParticle();
  if (!ChainCheck())
  {
    return particle;
  }

  if (ipart >= nh)
  {
    return particle;
  }

  particle->SetEnergy(ene[ipart]);
  particle->SetPdg(pdg[ipart]);
  particle->SetPxPyPz(momx[ipart], momy[ipart], momz[ipart]);
  particle->SetTime(0.);
  particle->SetXYZ(0., 0., 0.);
  particle->SetCharge(mciUtility::GetInstance()->GetCharge(pdg[ipart]));

  return particle;
}
