#include <mciWriter_mcpico.h>

ClassImp(mciWriter_mcpico);

mciWriter_mcpico::mciWriter_mcpico(/* args */) : fOutFile(nullptr)
{
  fTree = new TTree(fTreeName, "Hadrons tree from MC RHIC models");
  fTree->Branch("bimp",&bimp,"bimp/F"); // impact parametr
  fTree->Branch("phi2",&phi2,"phi2/F"); // phiRP2
  fTree->Branch("phi3",&phi3,"phi3/F"); // phiRP3
  fTree->Branch("ecc2",&ecc2,"ecc2/F"); // eccentricity2
  fTree->Branch("ecc3",&ecc3,"ecc3/F"); // eccentricity3
  fTree->Branch("npart",&npart,"npart/I"); // number of participants
  fTree->Branch("nh",&nh,"nh/I");  // number of particles
  fTree->Branch("momx",&momx,"momx[nh]/F");
  fTree->Branch("momy",&momy,"momy[nh]/F");
  fTree->Branch("momz",&momz,"momz[nh]/F");
  fTree->Branch("ene",&ene,"ene[nh]/F");//[energy]
  fTree->Branch("hid",&hid,"hid[nh]/I");//[histrory id]
  fTree->Branch("pdg",&pdg,"pdg[nh]/I");//[particle data group code]
  fTree->Branch("charge",&charge,"charge[nh]/S");//[electric charge]
}

mciWriter_mcpico::~mciWriter_mcpico()
{
  delete fTree;
  delete fOutFile;
}

void mciWriter_mcpico::SetFile(const TString &outputFileName)
{
  if (!fOutFile) fOutFile = new TFile(outputFileName.Data(), "recreate");
}

void mciWriter_mcpico::SetEvent(mciEvent *const& _ievent)
{
  bimp  = _ievent->GetB();
  phi2  = _ievent->GetPhiRP();
  phi3  = 0.;
  ecc2  = 0.;
  ecc3  = 0.;
  npart = _ievent->GetNparticipants();
  nh    = _ievent->GetNparticles();
}

void mciWriter_mcpico::SetParticle(mciParticle *const& _iparticle, Int_t id)
{
  momx[id]   = _iparticle->GetPx();
  momy[id]   = _iparticle->GetPy();
  momz[id]   = _iparticle->GetPz();
  ene[id]    = _iparticle->GetEnergy();
  pdg[id]    = _iparticle->GetPdg();
  hid[id]    = 0.;
  charge[id] = _iparticle->GetCharge();
}

void mciWriter_mcpico::Fill()
{
  fTree->Fill();
}

void mciWriter_mcpico::Write()
{
  fOutFile->cd();
  fTree->Write();
}

void mciWriter_mcpico::Close()
{
  fOutFile->Close();
}